@@ -7,12 +7,16 @@ pymwp: implementation of MWP analysis on C code in Python.
 
 __title__ = "pymwp"
 __author__ = "Clément Aubert, Thomas Rubiano, Neea Rusch, Thomas Seiller"
-__license__ = "CC BY-NC 4.0"
+__license__ = "GPLv3"
+__version__ = "0.3.0"
 
+from pymwp.parser import Parser
 from pymwp.delta_graphs import DeltaGraph
-from pymwp.version import __version__
+from pymwp.choice import Choices
 from pymwp.relation_list import RelationList
 from pymwp.relation import Relation
 from pymwp.polynomial import Polynomial
 from pymwp.monomial import Monomial
+from pymwp.result import Result
 from pymwp.analysis import Analysis
+
