@@ -1,14 +1,12 @@
+# noinspection DuplicatedCode
 import logging
 from typing import List, Tuple, Optional, Union, Dict
-from pycparser import c_ast
-from pycparser.c_ast import Node, Assignment, If, While, For, Compound, \
-    ParamList, FuncCall
 
-from .relation_list import RelationList, Relation
-from .polynomial import Polynomial
-from .monomial import Monomial
-from .delta_graphs import DeltaGraph
 from .file_io import save_relation
+# noinspection PyPep8Naming
+from .parser import Parser as pr
+from pymwp import DeltaGraph, Monomial, Polynomial, RelationList, Result
+from pymwp.result import FUNC_RESULT
 
 logger = logging.getLogger(__name__)
 
@@ -17,29 +15,27 @@ class Analysis:
     """MWP analysis implementation."""
 
     @staticmethod
-    def run(
-            ast: c_ast, file_out: str = None,
-            no_save: bool = False, no_eval: bool = False
-    ) -> Union[Dict, Tuple[Relation, List[List[int]], bool]]:
+    def run(ast: pr.AST, result: Result = None, **kwargs) \
+            -> Union[Dict[str, FUNC_RESULT], FUNC_RESULT]:
         """Run MWP analysis on specified input file.
 
         Arguments:
             ast: parsed C source code AST
-            file_out: where to store result
-            no_save: Set true when analysis result should not be saved to file
-            no_eval: Skip evaluation phase
+            result: (optional) result object
 
         Returns:
-              - Computed relation,
-              - list of non-infinity choices
-              - infinite/not infinite (boolean flag)
+            Computed relation, list of non-infinity choices, and
+                infinite/not infinite (boolean flag)
         """
+        file_out: str = kwargs['file_out'] if 'file_out' in kwargs else None
+        save: bool = 'no_save' not in kwargs or kwargs['no_save'] is False
+        stop_early: bool = 'fin' not in kwargs or kwargs['fin'] is False
+        skip_eval: bool = 'no_eval' in kwargs and kwargs['no_eval'] is True
+        result__: Result = result or Result()
 
-        logger.debug("starting analysis")
-        single_function = len(ast.ext) == 1
-        result, function_name = {}, ''
-
-        for ast_ext in ast:
+        logger.debug("started analysis")
+        result__.on_start()
+        for ast_ext in [f for f in ast if pr.is_func(f)]:
             choices = [0, 1, 2]
             index, combinations = 0, []
             function_name = ast_ext.decl.name
@@ -58,44 +54,41 @@ class Analysis:
                 logger.debug(f'computing relation...{i} of {total}')
                 index, rel_list, delta_infty = Analysis \
                     .compute_relation(index, node, dg)
-                if delta_infty:
+                if stop_early and delta_infty:
                     break
                 logger.debug(f'computing composition...{i} of {total}')
                 relations.composition(rel_list)
 
-            # skip evaluation when delta graph has detected infinity
-            # or caller has manually disabled evaluation
-            if not delta_infty and not no_eval:
-                combinations = relations.first.non_infinity(choices, index, dg)
+            # evaluate unless not enforcing finish and delta-infty
+            if not skip_eval and not delta_infty:
+                combinations = relations.first.eval(choices, index)
                 evaluated = True
 
             # the evaluation is infinite when either of these conditions holds:
             infinite = delta_infty or (
                     relations.first.variables and index > 0 and
-                    (evaluated and not combinations))
+                    evaluated and not combinations.valid)
 
             # record and display results
-            if infinite:
-                result[function_name] = None, None, True
-                logger.info(f'RESULT: {function_name} is infinite')
+            outcome = function_name,
+            if infinite and stop_early:
+                outcome += (None, None, True)
+            elif infinite:
+                outcome += (relations.first, None, True)
             else:
-                result[function_name] = relations.first, combinations, False
-                logger.info(f'\nMATRIX{relations}')
-                if not evaluated:
-                    logger.info('Skipped evaluation')
-                else:
-                    logger.info(f'CHOICES:\n{combinations}')
+                outcome += (relations.first, combinations, False)
+            result__.add_relation(*outcome)
 
-        # save result to file unless explicitly disabled
-        if not no_save:
-            save_relation(file_out, result)
+        result__.on_end()
+        result__.log_result()
 
-        # return results to caller
-        return result[function_name] if single_function else result
+        if save:
+            save_relation(file_out, result__.relations)
+        return result__.get_result()
 
     @staticmethod
     def find_variables(
-            function_body: Compound, param_list: Optional[ParamList]
+            function_body: pr.Compound, param_list: Optional[pr.ParamList]
     ) -> List[str]:
         """Finds all local variable declarations in function body and
         parameter list.
@@ -117,7 +110,7 @@ class Analysis:
 
         def recurse_nodes(node_):
             # only look for declarations
-            if isinstance(node_, c_ast.Decl):
+            if isinstance(node_, pr.Decl):
                 variables.append(node_.name)
             if hasattr(node_, 'block_items'):
                 for sub_node in node_.block_items:
@@ -136,7 +129,7 @@ class Analysis:
         return variables
 
     @staticmethod
-    def compute_relation(index: int, node: Node, dg: DeltaGraph) \
+    def compute_relation(index: int, node: pr.Node, dg: DeltaGraph) \
             -> Tuple[int, RelationList, bool]:
         """Create a relation list corresponding for all possible matrices
         of an AST node.
@@ -152,28 +145,28 @@ class Analysis:
 
         logger.debug("in compute_relation")
 
-        if isinstance(node, c_ast.Decl):
+        if isinstance(node, pr.Decl):
             return index, RelationList(), False
-        if isinstance(node, FuncCall):
+        if isinstance(node, pr.FuncCall):
             return Analysis.func_call(index)
-        if isinstance(node, c_ast.Assignment):
-            if isinstance(node.rvalue, c_ast.BinaryOp):
+        if isinstance(node, pr.Assignment):
+            if isinstance(node.rvalue, pr.BinaryOp):
                 return Analysis.binary_op(index, node)
-            if isinstance(node.rvalue, c_ast.Constant):
+            if isinstance(node.rvalue, pr.Constant):
                 return Analysis.constant(index, node.lvalue.name)
-            if isinstance(node.rvalue, c_ast.UnaryOp):
+            if isinstance(node.rvalue, pr.UnaryOp):
                 return Analysis.unary_op(index, node)
-            if isinstance(node.rvalue, c_ast.ID):
+            if isinstance(node.rvalue, pr.ID):
                 return Analysis.id(index, node)
-            if isinstance(node.rvalue, FuncCall):
+            if isinstance(node.rvalue, pr.FuncCall):
                 return Analysis.func_call(index)
-        if isinstance(node, c_ast.If):
+        if isinstance(node, pr.If):
             return Analysis.if_(index, node, dg)
-        if isinstance(node, c_ast.While):
+        if isinstance(node, pr.While):
             return Analysis.while_(index, node, dg)
-        if isinstance(node, c_ast.For):
+        if isinstance(node, pr.For):
             return Analysis.for_(index, node, dg)
-        if isinstance(node, c_ast.Compound):
+        if isinstance(node, pr.Compound):
             return Analysis.compound_(index, node, dg)
 
         logger.debug(f"uncovered case! type: {type(node)}")
@@ -181,7 +174,7 @@ class Analysis:
         return index, RelationList(), False
 
     @staticmethod
-    def id(index: int, node: Assignment) \
+    def id(index: int, node: pr.Assignment) \
             -> Tuple[int, RelationList, bool]:
         """Analyze x = y (with x != y) and y not a const
 
@@ -194,7 +187,7 @@ class Analysis:
         """
         x = node.lvalue.name
         y = node.rvalue.name
-        if x == y or isinstance(node.rvalue, c_ast.Constant):
+        if x == y or isinstance(node.rvalue, pr.Constant):
             return index, RelationList(), False
 
         logger.debug('Computing Relation x = y')
@@ -223,7 +216,7 @@ class Analysis:
         return index + 1, rel_list, False
 
     @staticmethod
-    def binary_op(index: int, node: Assignment) \
+    def binary_op(index: int, node: pr.Assignment) \
             -> Tuple[int, RelationList, bool]:
         """Analyze binary operation, e.g. `x = y + z`.
 
@@ -255,14 +248,14 @@ class Analysis:
     @staticmethod
     def constant(index: int, variable_name: str) \
             -> Tuple[int, RelationList, bool]:
-        """Analyze a constant assignment of form: x = c where x is some
+        """Analyze a constant assignment of form `x = c` where x is some
         variable and c is constant.
 
-        From MWP paper:
+        !!! quote "From MWP paper:"
 
-        > To deal with constants, just replace the program’s constants by
-          variables and regard the replaced constants as input to these
-          variables.
+            To deal with constants, just replace the program’s constants by
+            variables and regard the replaced constants as input to these
+            variables.
 
         Arguments:
             index: delta index
@@ -275,7 +268,7 @@ class Analysis:
         return index, RelationList([variable_name]), False
 
     @staticmethod
-    def unary_op(index: int, node: Assignment) \
+    def unary_op(index: int, node: pr.Assignment) \
             -> Tuple[int, RelationList, bool]:
         """Analyze unary operator.
 
@@ -288,13 +281,11 @@ class Analysis:
         """
         logger.debug('Computing Relation (third case / unary)')
         var_name = node.lvalue.name
-        # list_var = None  # list_var(exp)
-        # variables = [var_name] + list_var
         variables = [var_name]
         return index, RelationList.identity(variables), False
 
     @staticmethod
-    def if_(index: int, node: If, dg: DeltaGraph) \
+    def if_(index: int, node: pr.If, dg: DeltaGraph) \
             -> Tuple[int, RelationList, bool]:
         """Analyze an if statement.
 
@@ -323,7 +314,8 @@ class Analysis:
 
     @staticmethod
     def if_branch(
-            node: If, index: int, relation_list: RelationList, dg: DeltaGraph
+            node: pr.If, index: int, relation_list: RelationList,
+            dg: DeltaGraph
     ) -> Tuple[int, bool]:
         """Analyze `if` or `else` branch of a conditional statement.
 
@@ -362,7 +354,7 @@ class Analysis:
         return index, False
 
     @staticmethod
-    def while_(index: int, node: While, dg: DeltaGraph) \
+    def while_(index: int, node: pr.While, dg: DeltaGraph) \
             -> Tuple[int, RelationList, bool]:
         """Analyze while loop.
 
@@ -387,21 +379,18 @@ class Analysis:
         logger.debug('while loop fixpoint')
         relations.fixpoint()
         relations.while_correction(dg)
-
         dg.fusion()
 
         exit_ = False
         if 0 in dg.graph_dict:
             if dg.graph_dict[0] == {(): {}}:
-                logger.info(f'delta graph:\n{dg}')
-                logger.info('delta_graphs: infinite')
-                logger.info('Exit now !')
+                logger.info('delta_graphs: infinite -> Exit now')
                 exit_ = True
 
         return index, relations, exit_
 
     @staticmethod
-    def for_(index: int, node: For, dg: DeltaGraph) \
+    def for_(index: int, node: pr.For, dg: DeltaGraph) \
             -> Tuple[int, RelationList, bool]:
         """Analyze for loop node.
 
@@ -430,7 +419,7 @@ class Analysis:
         return index, relations, False
 
     @staticmethod
-    def compound_(index: int, node: Compound, dg: DeltaGraph) \
+    def compound_(index: int, node: pr.Compound, dg: DeltaGraph) \
             -> Tuple[int, RelationList, bool]:
         """Compound AST node contains zero or more children and is
         created by braces in source code.
