@@ -1,11 +1,14 @@
 # flake8: noqa: W605
 
 from __future__ import annotations
-from typing import Optional, List, Tuple
-from .constants import SetInclusion
+from typing import List, Optional, Tuple, Union
 
+from .constants import SetInclusion
 from .semiring import ZERO_MWP, UNIT_MWP, prod_mwp, sum_mwp
 
+DELTA = Tuple[int, int]
+"""delta type is tuple of two int values."""
+
 
 class Monomial:
     """
@@ -16,8 +19,8 @@ class Monomial:
 
     Deltas are coded as pairs $(i,j)$ with:
 
-     - $i$ the value and
-     - $j$ the index in the domain (infinite product)
+    - $i$ the value and
+    - $j$ the index in the domain (infinite product)
 
     We will have that $(i,j)$ will be equal to the unit of the semi-ring
     iff the $j^{th}$ input is equal to $i$ (so, the $j^{th}$ choice is $i$).
@@ -26,7 +29,9 @@ class Monomial:
     and no two deltas can have the same index.
     """
 
-    def __init__(self, scalar: str = UNIT_MWP, deltas: Optional[List[Tuple[int, int]]] = None):
+    def __init__(self, scalar: str = UNIT_MWP,
+                 deltas: Optional[Union[List[DELTA], DELTA]] = None,
+                 *args: Optional[DELTA]):
         """Create a monomial.
 
         Example:
@@ -47,19 +52,23 @@ class Monomial:
         Create monomial with scalar $w$ and two deltas
 
         ```python
-        mono = Monomial('w', [(0, 0), (1, 1)]
+        mono = Monomial('w', (0, 0), (1, 1))
         ```
 
         Arguments:
             scalar: monomial scalar
-            deltas: list of deltas
+            deltas: list of deltas - either a syntactic list,
+                or sequence of tuples that represent deltas.
         """
-
         self.deltas = []
         self.scalar = scalar
 
-        if deltas:
-            Monomial.insert_deltas(self, deltas)
+        if deltas is not None:
+            if isinstance(deltas, list):
+                Monomial.insert_deltas(self, deltas)
+            if isinstance(deltas, tuple):
+                delta_list = [deltas] + list(args if args else [])
+                Monomial.insert_deltas(self, delta_list)
 
     def __str__(self) -> str:
         deltas = [".delta({0},{1})".format(*delta)
@@ -73,7 +82,6 @@ class Monomial:
         """check if all deltas of m are in deltas of self
 
         Arguments:
-            self: this monomial
             m: a monomial to search for intersection
 
         Returns:
@@ -89,25 +97,24 @@ class Monomial:
         """gives info about inclusion of self monomial with monomial
 
         Arguments:
-            self: this monomial
             monomial: a monomial to see inclusion
 
         Returns:
             CONTAINS if self contains monomial
-            INCLUDED if self is included in monomial
-            EMPTY none of them
+            INCLUDED if self is included in monomial
+            EMPTY none of them
         """
         # self contains monomial ?
-        contains:bool = self.contains(monomial)
+        contains: bool = self.contains(monomial)
 
-        summ = sum_mwp(self.scalar,monomial.scalar)
+        summ = sum_mwp(self.scalar, monomial.scalar)
         # if self contains monomial and self.scalar >= monomial.scalar
         if contains and (monomial.scalar == summ):
             return SetInclusion.CONTAINS
         else:
             # self included in monomial and self.scalar <= monomial.scalar
-            included:bool = monomial.contains(self)
-            if included and (self.scalar==summ):
+            included: bool = monomial.contains(self)
+            if included and (self.scalar == summ):
                 return SetInclusion.INCLUDED
             else:
                 return SetInclusion.EMPTY
@@ -151,39 +158,6 @@ class Monomial:
 
         return mono_product
 
-    def eval(self, argument_list: List[int]) -> str:
-        """Evaluate delta values against argument list.
-
-        The result of eval is determined as follows:
-
-        If the list of values given as an argument "match" all
-        the deltas, then the value is returned. Otherwise 0 is
-        returned.
-
-        When matching we compare delta value $i$ at index $j$ to the
-        $j^{th}$ value in the argument list.
-
-        It can accommodate list of values that are of length
-        greater than the max of the indices in the deltas, but
-        not a list whose length is shorter than the max of the
-        indices in the delta.
-
-        Arguments:
-            argument_list: list of deltas to evaluate
-
-        Raises:
-            IndexError: if argument list length is less than
-                max index in the list of deltas.
-
-        Returns:
-            Scalar of the monomial if the evaluation matches and
-            otherwise 0.
-        """
-        for (i, j) in self.deltas:
-            if argument_list[j] != i:
-                return ZERO_MWP
-        return self.scalar
-
     def copy(self) -> Monomial:
         """Make a deep copy of a monomial."""
         return Monomial(self.scalar, self.deltas[:])
@@ -200,7 +174,7 @@ class Monomial:
         }
 
     @staticmethod
-    def insert_deltas(monomial: Monomial, deltas: List[tuple]) -> None:
+    def insert_deltas(monomial: Monomial, deltas: List[DELTA]) -> None:
         """Insert new deltas into monomial list of deltas.
 
         Arguments:
@@ -224,7 +198,7 @@ class Monomial:
                 break
 
     @staticmethod
-    def insert_delta(sorted_deltas: List[tuple], delta: tuple) -> List[tuple]:
+    def insert_delta(sorted_deltas: List[DELTA], delta: DELTA) -> List[DELTA]:
         """
         Takes as input a _sorted_ list of deltas and a delta.
 
