@@ -31,7 +31,10 @@ class Polynomial:
     iff $\\delta(i_1,j_1) < \\delta(m_1,n_1)$.
     """
 
-    def __init__(self, monomials: Optional[Union[str, List[Monomial]]] = None):
+    def __init__(
+            self,
+            monomials: Optional[Union[str, List[Monomial], Monomial]] = None,
+            *args: Optional[Monomial]):
         """Create a polynomial.
 
         Example:
@@ -47,13 +50,13 @@ class Polynomial:
         ```python
         poly = Polynomial('w')               # shorthand
 
-        poly = Polynomial([Monomial('w')])   # longer, equivalent
+        poly = Polynomial(Monomial('w'))     # longer, equivalent
         ```
 
         Create polynomial with two monomials and deltas
 
         ```python
-        poly = Polynomial([Monomial('m', [(0, 1)]), Monomial('w', [(1, 1)])])
+        poly = Polynomial(Monomial('m', (0, 1)), Monomial('w', (1, 1)))
         ```
 
         Arguments:
@@ -61,6 +64,8 @@ class Polynomial:
         """
         if monomials is not None and isinstance(monomials, str):
             self.list = [Monomial(monomials)]
+        elif monomials is not None and isinstance(monomials, Monomial):
+            self.list = [monomials] + list(args if args else [])
         else:
             self.list = monomials or [Monomial(ZERO_MWP)]
 
@@ -77,6 +82,12 @@ class Polynomial:
     def __mul__(self, other):
         return self.times(other)
 
+    @property
+    def eval(self):
+        """List of monomial deltas whose scalar is infinity."""
+        return [tuple(mono.deltas) for mono in
+                [m for m in self.list if m.scalar == 'i']]
+
     @staticmethod
     def inclusion(list_monom: list, mono: Monomial, i: int = 0) \
             -> Tuple[bool, int]:
@@ -287,26 +298,6 @@ class Polynomial:
 
         return Polynomial(result).remove_zeros()
 
-    def eval(self, argument_list: list[int]) -> str:
-        """Evaluate polynomial.
-
-        This method performs map() on each monomial against the argument list
-        then reduces the result to a single result determined by
-        [`semiring#sum_mwp`](semiring.md#pymwp.semiring.sum_mwp) function.
-
-        Arguments:
-            argument_list: list of indices of deltas to evaluate
-
-        Returns:
-            scalar value
-        """
-        result = ZERO_MWP
-        for monomial in self.list:
-            result = sum_mwp(result, monomial.eval(argument_list))
-            if result == 'i':
-                break
-        return result
-
     def equal(self, polynomial: Polynomial) -> bool:
         """Determine if two polynomials are equal.
 
