@@ -1,7 +1,8 @@
 from pymwp import Analysis, Polynomial
 from .mocks.ast_mocks import \
-    INFINITE_2C, NOT_INFINITE_2C, IF_WO_BRACES, IF_WITH_BRACES, \
-    VARIABLE_IGNORED, BRACES_ISSUES, PARAMS, FUNCTION_CALL, INFINITE_8C
+    INFINITE_2C, INFINITE_8C, NOT_INFINITE_2C, NOT_INFINITE_3C, \
+    IF_WO_BRACES, IF_WITH_BRACES, VARIABLE_IGNORED, BRACES_ISSUES, \
+    PARAMS, FUNCTION_CALL, EMPTY
 
 
 def test_analyze_infinite2():
@@ -13,6 +14,12 @@ def test_analyze_infinite2():
     assert relation is None  # no relation since infinite
 
 
+def test_analyze_infinite_8():
+    """Check analysis result for infinite/infinite_8.c"""
+    _, _, infty = Analysis.run(INFINITE_8C, no_save=True)
+    assert infty
+
+
 def test_analyze_non_infinite_2():
     """Check analysis result for not_infinite/notinfinite_2.c"""
     relation, combinations, infty = Analysis.run(NOT_INFINITE_2C, no_save=True)
@@ -21,9 +28,15 @@ def test_analyze_non_infinite_2():
 
     # match expected choices and variables
     assert set(relation.variables) == {'X0', 'X1'}
-    assert combinations == [[0, 0], [0, 1], [0, 2],
-                            [1, 0], [1, 1], [1, 2],
-                            [2, 0], [2, 1], [2, 2]]
+    assert combinations.is_valid(0, 0)
+    assert combinations.is_valid(0, 1)
+    assert combinations.is_valid(0, 2)
+    assert combinations.is_valid(1, 0)
+    assert combinations.is_valid(1, 1)
+    assert combinations.is_valid(1, 2)
+    assert combinations.is_valid(2, 0)
+    assert combinations.is_valid(2, 1)
+    assert combinations.is_valid(2, 2)
 
     # match *some* deltas from the matrix
     assert str(relation.matrix[0][0].list[0]) == 'w.delta(0,0)'
@@ -31,10 +44,19 @@ def test_analyze_non_infinite_2():
     assert str(relation.matrix[0][0].list[2]) == 'w.delta(2,0)'
 
 
-def test_analyze_infinite_8():
-    """Check analysis result for infinite/infinite_8.c"""
-    _, _, infty = Analysis.run(INFINITE_8C, no_save=True)
-    assert infty
+def test_analyze_non_infinite_3():
+    """Check analysis result for not_infinite/notinfinite_3.c"""
+    _, combinations, infty = Analysis.run(NOT_INFINITE_3C, no_save=True)
+
+    assert not infty
+    assert len(combinations.valid) == 1
+    assert combinations.valid[0] == [[0, 1, 2], [0, 1, 2], [2]]
+
+
+def test_analyze_infinite_to_completion():
+    """Check analysis completion for infinite program"""
+    matrix, _, _ = Analysis.run(INFINITE_2C, no_save=True, fin=True)
+    assert matrix
 
 
 def test_analyze_if_braces_do_not_matter():
@@ -42,27 +64,32 @@ def test_analyze_if_braces_do_not_matter():
      should give the same analysis result."""
     rel_with, choices_with = Analysis.run(IF_WITH_BRACES, no_save=True)[:2]
     rel_wo, choices_wo = Analysis.run(IF_WO_BRACES, no_save=True)[:2]
+    all_valid_choices = [
+        [0, 0, 0], [0, 0, 1], [0, 0, 2], [0, 1, 0], [0, 1, 1],
+        [0, 1, 2], [0, 2, 0], [0, 2, 1], [0, 2, 2], [1, 0, 0],
+        [1, 0, 1], [1, 0, 2], [1, 1, 0], [1, 1, 1], [1, 1, 2],
+        [1, 2, 0], [1, 2, 1], [1, 2, 2], [2, 0, 0], [2, 0, 1],
+        [2, 0, 2], [2, 1, 0], [2, 1, 1], [2, 1, 2], [2, 2, 0],
+        [2, 2, 1], [2, 2, 2]]
 
     # match choices and variables
     assert set(rel_with.variables) == set(rel_wo.variables) == \
            {'x', 'x1', 'x2', 'x3', 'y'}
-    # should give same choices
-    assert choices_with == choices_wo == \
-           [[0, 0, 0], [0, 0, 1], [0, 0, 2], [0, 1, 0], [0, 1, 1],
-            [0, 1, 2], [0, 2, 0], [0, 2, 1], [0, 2, 2], [1, 0, 0],
-            [1, 0, 1], [1, 0, 2], [1, 1, 0], [1, 1, 1], [1, 1, 2],
-            [1, 2, 0], [1, 2, 1], [1, 2, 2], [2, 0, 0], [2, 0, 1],
-            [2, 0, 2], [2, 1, 0], [2, 1, 1], [2, 1, 2], [2, 2, 0],
-            [2, 2, 1], [2, 2, 2]]
+    # both results accept same & all choices
+    for choice in all_valid_choices:
+        assert choices_with.is_valid(*choice) and choices_wo.is_valid(*choice)
 
 
 def test_analyze_variable_ignore():
     """Analysis picks up variable on left of assignment,
     see issue #11: https://github.com/statycc/pymwp/issues/11 """
     relation, combinations = Analysis.run(VARIABLE_IGNORED, no_save=True)[:2]
-
-    assert combinations == [[0, 0], [0, 1], [0, 2], [1, 0], [1, 1], [1, 2],
+    non_infinity_choices = [[0, 0], [0, 1], [0, 2],
+                            [1, 0], [1, 1], [1, 2],
                             [2, 0], [2, 1], [2, 2]]
+
+    for choice in non_infinity_choices:
+        assert combinations.is_valid(*choice)
     assert set(relation.variables) == {'X2', 'X3', 'X1', 'X4'}
 
     mpw = '+m.delta(0,0)+p.delta(1,0)+w.delta(2,0)'
@@ -117,3 +144,8 @@ def test_analysis_returns_all_functions():
 
     assert not f_infty
     assert set(foo.variables) == {'X1', 'X2'}
+
+
+def test_analysis_handles_empty_program():
+    relation = Analysis.run(EMPTY, no_save=True)
+    assert relation == {}
